import math, copy, worldclass, packageclass
from constants import *
def getlength(a, b=[0,0]):
	return math.sqrt( (b[0] - a[0])**2 + (b[1] - a[1])**2 )

class Rope:
	def __init__(self, end1, end2, numparticles, gravity, timestep):
		self.gravity = gravity
		length = getlength(end1, end2)
		self.stickleng = float(length)/float(numparticles-1)	#we have a particle on each end
		addval = [(float(end2[0] - end1[0])/float(numparticles-1)), (float(end2[1] - end1[1])/float(numparticles-1))]
		self.particles = []
		self.oldparticles = []
		self.forces = []
		self.invmasses = []
		self.numparticles = numparticles
		self.timestep = timestep
		self.keyleft = False
		self.keyright = False
		for i in range(numparticles):
			self.particles.append([end1[0]+addval[0]*i,end1[1]+addval[1]*i])
			self.oldparticles.append([end1[0]+addval[0]*i,end1[1]+addval[1]*i])
			self.forces.append([0,0])
			self.invmasses.append(1/float(ropepartmass))
		self.invmasses[0] = 0.002
	
	def go(self):
		for i in range(self.numparticles):
			temp = [0,0]
			temp[0] = self.particles[i][0]
			temp[1] = self.particles[i][1]
			
			self.particles[i][0] += self.particles[i][0] - self.oldparticles[i][0] + (self.forces[i][0] * self.timestep**2)
			self.particles[i][1] += self.particles[i][1] - self.oldparticles[i][1] + (self.forces[i][1] * self.timestep**2)
			
			self.oldparticles[i][0] = temp[0]
			self.oldparticles[i][1] = temp[1]
			
	def accumulate(self, input, ship):
		for each in self.forces:
			each[1] = self.gravity
		if ship.strength > 0 and ship.fuel > 0:
			if input.keys["up"] == True:
				self.forces[0][1] -= thrusterstrength
				ship.fuel -= self.timestep * fueluseconstant
			
			if input.keys["left"] == True:
				if self.keyleft == False:
					self.keyleft = True
					self.forces[0][0] -= posthrusterstrength
				ship.fuel -= self.timestep * fueluseconstant * posthrusterstrength / thrusterstrength
				
			if input.keys["left"] == False:
				if self.keyleft == True:
					self.forces[0][0] += posthrusterstrength	
				self.keyleft = False
			
			if input.keys["right"] == True:
				if self.keyright == False:
					self.keyright = True
					self.forces[0][0] += posthrusterstrength
				ship.fuel -= self.timestep * fueluseconstant * posthrusterstrength / thrusterstrength
				
			if input.keys["right"] == False:
				if self.keyright == True:
					self.forces[0][0] -= posthrusterstrength
				self.keyright = False
			
	def satcons(self, ship, world, carrying, maingame, sounds):		#satisfy constraints
		ship.setpos(self.particles[0])
		a = None
		aa = ship.collision()
		
		if aa <> None:
			a1 = aa[0]
			a = aa[1]

		if a <> None and a1 == 0:	#embedded in floor
			speed = getlength(self.particles[0],self.oldparticles[0])
			if world.islandingpad(self.particles[0][0]) and (speed < 10):
				self.oldparticles[0][0] = self.particles[0][0]
				if ship.strength < ship.maxstrength and maingame.money > 0:
					ship.strength += shiprecharge * self.timestep
					maingame.money -= rechargecost * self.timestep
				if ship.fuel < ship.maxfuel and maingame.money > 0:
					ship.fuel += shiprefuel * self.timestep
					maingame.money -= refuelcost * self.timestep
			else:
				#sounds.play('hit')
				ship.strength -= penaltyconstant * speed
				if ship.strength < 0:
					ship.strength = 0
					
		if a <> None and a1 == 1:#embedded in ceiling
			#sounds.play('hit')
			speed = getlength(self.particles[0],self.oldparticles[0])
			ship.strength -= penaltyconstant * speed
			if ship.strength < 0:
				ship.strength = 0
		
		if carrying <> None:
			h = world.checkfloor(self.particles[-1][0])
			h2 = world.checkceil(self.particles[-1][0])
			if self.particles[-1][1] > h:
				speed = getlength(self.particles[-1],self.oldparticles[-1])
				if world.islandingpad(self.particles[-1][0]) and (speed < 10):
					self.oldparticles[-1][0] = self.particles[-1][0]
				else:
					#sounds.play('hit')
					if carrying.hasbeenairborne == True:
						carrying.strength -= penaltyconstant * speed
					b = world.getprojection(self.particles[-1])
					self.particles[-1][0] = b[0]
					self.particles[-1][1] = b[1]
			elif self.particles[-1][1] < h-3:
				carrying.hasbeenairborne = True
			
			if self.particles[-1][1] < h2:	#ok this is if we hit the ceiling
				#sounds.play('hit')
				speed = getlength(self.particles[-1],self.oldparticles[-1])
				carrying.strength -= penaltyconstant * speed
				b = world.GetCeilProjection(self.particles[-1])
				self.particles[-1][0] = b[0]
				self.particles[-1][1] = b[1]
		for z in range(numiterations):
			
			if a != None:		
				self.particles[0][0] = a[0]
				self.particles[0][1] = a[1]
				#self.particles[0][0] -= frictionConstant * (self.particles[0][0] - self.oldparticles[0][0])
				#ship.strength -= penaltyconstant * (self.particles[0][0] - self.oldparticles[0][0]) / numiterations
			
			#keeps within bounds of window ( change this to keep within the world(i.e. don't go through walls)
			for each in self.particles:
					
				
					
				if each[0] > (worldsize-1)*100 - halfwindowsize[0]:
					each[0] = (worldsize-1)*100 - halfwindowsize[0]

				if each[0] < halfwindowsize[0]:
					each[0] = halfwindowsize[0]

			for i in range(len(self.particles)):
				if self.particles[i][1] > world.checkfloor(self.particles[i][0]):
					self.particles[i][1] = world.checkfloor(self.particles[i][0])				
					self.particles[i][0] -= frictionConstant * (self.particles[i][0] - self.oldparticles[i][0])
				if self.particles[i][1] < world.checkceil(self.particles[i][0]):
					self.particles[i][1] = world.checkceil(self.particles[i][0])				
					self.particles[i][0] -= frictionConstant * (self.particles[i][0] - self.oldparticles[i][0])
		
			#this section keeps the sticks rigid
			delta = [0,0]
			for i in range(self.numparticles-1):
				delta[0] = self.particles[i+1][0] - self.particles[i][0]
				delta[1] = self.particles[i+1][1] - self.particles[i][1]
				#deltalength = math.sqrt(delta[0]*delta[0] + delta[1]*delta[1])
				deltalength = getlength(delta)
				#deltalength=(self.stickleng**2)/((delta[0]*delta[0]+delta[1]*delta[1])+self.stickleng**2)-0.5;
				#delta[0] *= deltalength
				#delta[1] *= deltalength
				#print "deltalength", deltalength
				if deltalength != 0:
					diff = (deltalength-self.stickleng)/(deltalength*(self.invmasses[i] + self.invmasses[i+1]));
				else:
					diff = 0
				
				#print "diff", diff
				self.particles[i][0] += self.invmasses[i] * delta[0] * diff
				self.particles[i][1] += self.invmasses[i] * delta[1] * diff
				self.particles[i+1][0] -= self.invmasses[i+1] * delta[0] * diff
				self.particles[i+1][1] -= self.invmasses[i+1] * delta[1] * diff
		
		
	def dotimestep(self, ship, input, world, carry, maingame, sounds):	#endpos is the position you want the end of the rope in, so we can pass it into the constraints
		self.accumulate(input, ship)
		self.go()
		self.satcons(ship, world, carry, maingame, sounds)
		ship.setpos(self.particles[0])
	
	def setmass(self, mass, linknum=-1):
		self.invmasses[linknum] = 1/float(mass)
	
	
	def draw(self, graphics, acc):
		self.centerx = (self.oldparticles[0][0] + (self.particles[0][0] - self.oldparticles[0][0]) * acc/self.timestep)
		for i in range(self.numparticles-1):
			x1 = (self.oldparticles[i][0] + (self.particles[i][0] - self.oldparticles[i][0]) * acc/self.timestep) - self.centerx + halfwindowsize[0]
			y1 = (self.oldparticles[i][1] + (self.particles[i][1] - self.oldparticles[i][1]) * acc/self.timestep)
			x2 = (self.oldparticles[i+1][0] + (self.particles[i+1][0] - self.oldparticles[i+1][0]) * acc/self.timestep) - self.centerx + halfwindowsize[0]
			y2 = (self.oldparticles[i+1][1] + (self.particles[i+1][1] - self.oldparticles[i+1][1]) * acc/self.timestep)
			graphics.drawline((x1,y1),(x2,y2), 10);
